Class: KPeg::CompiledParser
- Inherits:
-
Object
- Object
- KPeg::CompiledParser
show all
- Includes:
- Position
- Defined in:
- lib/kpeg/compiled_parser.rb
Defined Under Namespace
Classes: LeftRecursive, MemoEntry, ParseError, RuleInfo
Instance Attribute Summary collapse
Class Method Summary
collapse
Instance Method Summary
collapse
Methods included from Position
#current_column, #current_line, #lines
Constructor Details
#initialize(str, debug = false) ⇒ CompiledParser
This is distinct from setup_parser so that a standalone parser can redefine #initialize and still have access to the proper parser setup code.
30
31
32
|
# File 'lib/kpeg/compiled_parser.rb', line 30
def initialize(str, debug=false)
setup_parser(str, debug)
end
|
Instance Attribute Details
#failed_rule ⇒ Object
Returns the value of attribute failed_rule.
128
129
130
|
# File 'lib/kpeg/compiled_parser.rb', line 128
def failed_rule
@failed_rule
end
|
#failing_rule_offset ⇒ Object
Returns the value of attribute failing_rule_offset.
35
36
37
|
# File 'lib/kpeg/compiled_parser.rb', line 35
def failing_rule_offset
@failing_rule_offset
end
|
#pos ⇒ Object
Returns the value of attribute pos.
36
37
38
|
# File 'lib/kpeg/compiled_parser.rb', line 36
def pos
@pos
end
|
#result ⇒ Object
Returns the value of attribute result.
35
36
37
|
# File 'lib/kpeg/compiled_parser.rb', line 35
def result
@result
end
|
#string ⇒ Object
Returns the value of attribute string.
34
35
36
|
# File 'lib/kpeg/compiled_parser.rb', line 34
def string
@string
end
|
Class Method Details
.rule_info(name, rendered) ⇒ Object
294
295
296
|
# File 'lib/kpeg/compiled_parser.rb', line 294
def self.rule_info(name, rendered)
RuleInfo.new(name, rendered)
end
|
Instance Method Details
#apply(rule) ⇒ Object
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
|
# File 'lib/kpeg/compiled_parser.rb', line 231
def apply(rule)
if m = @memoizations[rule][@pos]
m.inc!
prev = @pos
@pos = m.pos
if m.ans.kind_of? LeftRecursive
m.ans.detected = true
return nil
end
@result = m.result
return m.ans
else
lr = LeftRecursive.new(false)
m = MemoEntry.new(lr, @pos)
@memoizations[rule][@pos] = m
start_pos = @pos
ans = __send__ rule
m.move! ans, @pos, @result
if ans and lr.detected
return grow_lr(rule, start_pos, m)
else
return ans
end
return ans
end
end
|
#external_invoke(other, rule, *args) ⇒ Object
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
|
# File 'lib/kpeg/compiled_parser.rb', line 211
def external_invoke(other, rule, *args)
old_pos = @pos
old_string = @string
@pos = other.pos
@string = other.string
begin
if val = __send__(rule, *args)
other.pos = @pos
else
other.set_failed_rule "#{self.class}##{rule}"
end
val
ensure
@pos = old_pos
@string = old_string
end
end
|
#failure_caret ⇒ Object
65
66
67
68
69
70
71
|
# File 'lib/kpeg/compiled_parser.rb', line 65
def failure_caret
l = current_line @failing_rule_offset
c = current_column @failing_rule_offset
line = lines[l-1]
"#{line}\n#{' ' * (c - 1)}^"
end
|
#failure_character ⇒ Object
73
74
75
76
77
|
# File 'lib/kpeg/compiled_parser.rb', line 73
def failure_character
l = current_line @failing_rule_offset
c = current_column @failing_rule_offset
lines[l-1][c-1, 1]
end
|
#failure_info ⇒ Object
53
54
55
56
57
58
59
60
61
62
63
|
# File 'lib/kpeg/compiled_parser.rb', line 53
def failure_info
l = current_line @failing_rule_offset
c = current_column @failing_rule_offset
if @failed_rule.kind_of? Symbol
info = self.class::Rules[@failed_rule]
"line #{l}, column #{c}: failed rule '#{info.name}' = '#{info.rendered}'"
else
"line #{l}, column #{c}: failed rule '#{@failed_rule}'"
end
end
|
#failure_oneline ⇒ Object
79
80
81
82
83
84
85
86
87
88
89
90
91
|
# File 'lib/kpeg/compiled_parser.rb', line 79
def failure_oneline
l = current_line @failing_rule_offset
c = current_column @failing_rule_offset
char = lines[l-1][c-1, 1]
if @failed_rule.kind_of? Symbol
info = self.class::Rules[@failed_rule]
"@#{l}:#{c} failed rule '#{info.name}', got '#{char}'"
else
"@#{l}:#{c} failed rule '#{@failed_rule}', got '#{char}'"
end
end
|
#get_byte ⇒ Object
151
152
153
154
155
156
157
158
159
|
# File 'lib/kpeg/compiled_parser.rb', line 151
def get_byte
if @pos >= @string.size
return nil
end
s = @string.getbyte @pos
@pos += 1
s
end
|
#get_text(start) ⇒ Object
40
41
42
|
# File 'lib/kpeg/compiled_parser.rb', line 40
def get_text(start)
@string[start..@pos-1]
end
|
#grow_lr(rule, start_pos, m) ⇒ Object
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
|
# File 'lib/kpeg/compiled_parser.rb', line 267
def grow_lr(rule, start_pos, m)
while true
@pos = start_pos
@result = m.result
ans = __send__ rule
return nil unless ans
break if @pos <= m.pos
m.move! ans, @pos, @result
end
@result = m.result
@pos = m.pos
return m.ans
end
|
#match_string(str) ⇒ Object
130
131
132
133
134
135
136
137
138
|
# File 'lib/kpeg/compiled_parser.rb', line 130
def match_string(str)
len = str.size
if @string[pos,len] == str
@pos += len
return str
end
return nil
end
|
#parse(rule = nil) ⇒ Object
172
173
174
175
176
177
178
179
180
|
# File 'lib/kpeg/compiled_parser.rb', line 172
def parse(rule=nil)
if !rule
_root ? true : false
else
method = rule.gsub("-","_hyphen_")
__send__("_#{method}") ? true : false
end
end
|
#raise_error ⇒ Object
96
97
98
|
# File 'lib/kpeg/compiled_parser.rb', line 96
def raise_error
raise ParseError, failure_oneline
end
|
#scan(reg) ⇒ Object
140
141
142
143
144
145
146
147
148
|
# File 'lib/kpeg/compiled_parser.rb', line 140
def scan(reg)
if m = reg.match(@string[@pos..-1])
width = m.end(0)
@pos += width
return true
end
return nil
end
|
#set_failed_rule(name) ⇒ Object
121
122
123
124
125
126
|
# File 'lib/kpeg/compiled_parser.rb', line 121
def set_failed_rule(name)
if @pos > @failing_rule_offset
@failed_rule = name
@failing_rule_offset = @pos
end
end
|
#setup_foreign_grammar ⇒ Object
Must be outside the STANDALONE block because a standalone parser always injects it’s own version of this method.
8
9
|
# File 'lib/kpeg/compiled_parser.rb', line 8
def setup_foreign_grammar
end
|
#setup_parser(str, debug = false) ⇒ Object
Leave these markers in! They allow us to generate standalone code automatically!
STANDALONE START
15
16
17
18
19
20
21
22
23
24
|
# File 'lib/kpeg/compiled_parser.rb', line 15
def setup_parser(str, debug=false)
@string = str
@pos = 0
@memoizations = Hash.new { |h,k| h[k] = {} }
@result = nil
@failed_rule = nil
@failing_rule_offset = -1
setup_foreign_grammar
end
|
#show_error(io = STDOUT) ⇒ Object
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
|
# File 'lib/kpeg/compiled_parser.rb', line 100
def show_error(io=STDOUT)
error_pos = @failing_rule_offset
line_no = current_line(error_pos)
col_no = current_column(error_pos)
io.puts "On line #{line_no}, column #{col_no}:"
if @failed_rule.kind_of? Symbol
info = self.class::Rules[@failed_rule]
io.puts "Failed to match '#{info.rendered}' (rule '#{info.name}')"
else
io.puts "Failed to match rule '#{@failed_rule}'"
end
io.puts "Got: #{string[error_pos,1].inspect}"
line = lines[line_no-1]
io.puts "=> #{line}"
io.print(" " * (col_no + 3))
io.puts "^"
end
|
#show_pos ⇒ Object
44
45
46
47
48
49
50
51
|
# File 'lib/kpeg/compiled_parser.rb', line 44
def show_pos
width = 10
if @pos < width
"#{@pos} (\"#{@string[0,@pos]}\" @ \"#{@string[@pos,width]}\")"
else
"#{@pos} (\"... #{@string[@pos - width, width]}\" @ \"#{@string[@pos,width]}\")"
end
end
|